"""
setup script for the exos module
"""

from setuptools import setup

__author__ = 'Bruno Lange'
__email__ = 'blangeram@gmail.com'
__license__ = 'MIT'

setup(
    name='exos',
    version='0.1.1',
    packages=['exos',],
    description='EXpressions Over Statements: additional functional tools in Python!',
    author='Bruno Lange',
    author_email='blangeram@gmail.com',
    url='https://github.com/brunolange/exos',
    python_requires='>=3',
)
